/* Dies ist das Beispielprogramm xmdemo aus folgendem Buch:
 *
 *  Autoren: Hans-Joachim Brede, Nicolai Josuttis,
 *           Sabine Lemberg, Achim Loerke
 *  Titel:   Programmieren mit OSF/Motif Version 2
 *  Verlag:  Addison-Wesley, 2. voellig ueberarbeitete Auflage 1995
 *  ISBN:    3-89319-727-3
 *
 * Copyright 1995 by Hans-Joachim Brede, Nicolai Josuttis, Sabine Lemberg and
 * Achim Loerke.  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that the
 * names of Addison-Wesley or the authors are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Allgemeine C-Header-Dateien einbinden
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* Alle X- und Motif-Header-Dateien einbinden
 */
#include <Xm/XmAll.h>

/* Klassenname des Programms
 */
#define APPL_KLASSE    "XmDemo"

/* Konstante fuer App-Defaults-Version
 */
#define APP_DEFAULTS_VERSION    123

/* Exit-Konstanten (falls nicht in <stdlib.h> definiert)
 */
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS    0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE    1
#endif

/* Konstanten fuer die Groesse der Graphik
 */
#define GRAPHIK_BREITE    ((unsigned int)500)
#define GRAPHIK_HOEHE     ((unsigned int)500)


/* Globale Variablen
 */
XtAppContext kontext;              /* Applikations-Umgebung */
Display*     display;              /* Display */
Widget       graphikWidget;        /* Graphik-Widget */
Pixmap       graphikBitmap;        /* Bitmap fuer Graphik-Kopie */
String       graphikName = NULL;   /* aktueller Name der Graphik */
Widget       graphikNameWidget;    /* Widget fuer Graphik-Namen */
Widget       fsdWidget;            /* FileSelectionDialog zum Laden */

/* Attribute zum Malen
 */
Boolean      kontinuierlich;       /* kontinuierliches Malen ? */
int          malmodus;             /* Mal-Modus */
#define VG_MODUS   0               /*  mit Vordergrund-Farbe */
#define INV_MODUS  1               /*  invertierend */
#define HG_MODUS   2               /*  mit Hintergrund-Farbe */
#define MODUS_ANZ  3

/* GCs fuer Graphik-Fenster und Graphik-Bitmap
 */
GC fensterGC[MODUS_ANZ];     /* Fenster GCs */
GC bitmapGC[MODUS_ANZ];      /* Bitmap GCs */
GC copyGC;                   /* GC zum Kopieren von Bitmap ins Fenster */


/* Fallback-Ressourcen (nur das Notwendigste)
 */
static String fallbackRessourcen[] = {

   "*dateiname.labelString:       Dateiname: ",
   "*graphikName.alignment:       alignment_beginning",

   "*scrolledGraphik.scrollingPolicy:    automatic",
   "*scrolledGraphik.scrollBarPlacement: bottom_right",

   "*form.horizontalSpacing:             4",
   "*form.verticalSpacing:               4",
   "*graphikName.leftOffset:             0",
   "*form.backgroundPixmap:              25_foreground",
   "*scrolledGraphik.backgroundPixmap:   25_foreground",
   "*scrolledGraphik.XmDrawingArea.backgroundPixmap: 25_foreground",

   "*vordergrund.labelString:     Schwarz",
   "*invertieren.labelString:     Invers",
   "*hintergrund.labelString:     Weiss",
   "*breitenMenue.labelString:    Breite:",
   "*graphik.background:          white",
   "*graphik.foreground:          black",

   "*okLabelString:               OK",
   "*cancelLabelString:           Abbruch",
   "*hilfeDialog.messageString:   Keine Hilfe noetig",
   "*ladenDialog.dirMask:         *.xbm",
   "*speichernAlsDialog.selectionLabelString: Dateiname ?",

   NULL
};


/* Optionen
 */
static XrmOptionDescRec optionen[] = {

    /* Dateiname der zu editierenden Graphik-Bitmap
     */
    { "-datei", ".datei", XrmoptionSepArg, NULL },

#if XmVersion <= 1002
    /* '-font' und '-fn' setzen bis Motif 1.2 Font-Listen
     */
    { "-font", "*fontList", XrmoptionSepArg, NULL },
    { "-fn",   "*fontList", XrmoptionSepArg, NULL }
#endif
};


/* Konstanten fuer Applikations-Ressource definieren
 */
#define XbspNdatei               "datei"
#define XbspCDatei               "Datei"
#define XbspNappDefaultsVersion  "appDefaultsVersion"
#define XbspCAppDefaultsVersion  "AppDefaultsVersion"

/* Struktur-Datentyp fuer alle
 * Applikations-Ressourcen definieren
 */
typedef struct {
    String datei;      /* Appl.-Res. 'datei' */
    int    version;    /* Appl.-Res. 'appDefaultsVersion' */
} ApplResTyp;

/* Ressource-Liste mit allen Eigenschaften der
 * Applikations-Ressourcen definieren
 */
static XtResource ressourcen[] = {
  /* Name, Klasse, Typ, Size, Offset, Default-Typ, Default-Wert */

    /* Appl.-Ressource 'datei' (Default-Wert: "xmdemo.xbm")
     */
    { XbspNdatei, XbspCDatei,
      XmRString, sizeof(String),
      XtOffsetOf(ApplResTyp,datei),
      XmRString, "xmdemo.xbm" },

    /* Appl.-Ressource 'appDefaultsVersion' (Default-Wert: 0)
     */
    { XbspNappDefaultsVersion, XbspCAppDefaultsVersion,
      XmRInt, sizeof(int),
      XtOffsetOf(ApplResTyp,version),
      XmRString, "0" }
};



/*ARGSUSED*/
void beendenCB (widget, clientDaten, aufrufDaten)
Widget    widget;        /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Programm mit fehlerfreiem Status beenden
     */
    exit (EXIT_SUCCESS);
}


void initBeendenButton (elternWidget)
Widget elternWidget;
{
    Widget buttonW;

    /* Menuepunkt 'beenden' erzeugen
     */
    buttonW = XtCreateManagedWidget ("beenden", xmPushButtonWidgetClass,
                                     elternWidget, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNactivateCallback,
                   beendenCB, (XtPointer)NULL);
}


Boolean speicherGraphik (dateiname)
String dateiname;     /* Dateiname der abzuspeichernden Graphik */
{
    int status;          /* Speichern geklappt ? */

    /* Bitmap-Daten in 'dateiname' abspeichern
     */
    status = XWriteBitmapFile (display, dateiname, graphikBitmap,
                               GRAPHIK_BREITE, GRAPHIK_HOEHE, -1, -1);
    /* Hat das Speichern geklappt ?
     */
    if (status != BitmapSuccess) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Daten nicht abspeicherbar");
        return False;
    }

    return True;
}


/*ARGSUSED*/
void eingabeCB (widget, clientDaten, aufrufDaten)
Widget    widget;        /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten mit Callback-Informationen */
{
    XmSelectionBoxCallbackStruct* SBinfos;
    String                        dateiname;

    /* Aufruf-Daten der SelectionBox enthalten unter anderem
     * den eingegebenen Dateinamen
     */
    SBinfos = (XmSelectionBoxCallbackStruct*)aufrufDaten;

    /* Dateinamen auslesen (Kopie mit eigenem Speicherplatz)
     */
    XmStringGetLtoR (SBinfos->value, XmSTRING_DEFAULT_CHARSET,
                     &dateiname);

    /* Graphik dort abspeichern
     */
    speicherGraphik (dateiname);

    /* Speicherplatz fuer ausgelesenen Dateinamen wieder freigeben
     */
    XtFree (dateiname);
}


/*ARGSUSED*/
void speichernAlsCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    static Widget eingabeW = NULL;
    Arg           args[10];
    Cardinal      n;

    /* Falls noch nicht geschehen:
     *  XmPromptDialog zum Eingeben eines Dateinamens erzeugen
     *  (Rest des Programms wird bei Ausgabe gesperrt)
     */
    if (eingabeW == NULL) {
        n = 0;
        XtSetArg (args[n], XmNautoUnmanage, True);            n++;
        XtSetArg (args[n], XmNdialogStyle,
                           XmDIALOG_FULL_APPLICATION_MODAL);  n++;
        eingabeW = XmCreatePromptDialog (widget, "speichernAlsDialog",
                                         args, n);

        /* OK-Callback installieren
         */
        XtAddCallback (eingabeW, XmNokCallback,
                       eingabeCB, (XtPointer)NULL);
 
        /* Der Help-Button wird nicht gebraucht
         */
        XtUnmanageChild (XmSelectionBoxGetChild(eingabeW,
                                                XmDIALOG_HELP_BUTTON));
    }
 
    /* XmPromptDialog ausgeben
     */
    XtManageChild (eingabeW);
}


void initSpeichernAlsButton (elternWidget)
Widget elternWidget;
{
    Widget buttonW;

    /* XmPushButton zum Abspeichern einer Graphik
     * unter einem anderen Namen erzeugen
     */
    buttonW = XtCreateManagedWidget ("speichernAls",
                                     xmPushButtonWidgetClass,
                                     elternWidget, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNactivateCallback,
                   speichernAlsCB, (XtPointer)NULL);
}


/*ARGSUSED*/
void speichernCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Graphik abspeichern
     */
    speicherGraphik (graphikName);
}


void initSpeichernButton (elternWidget)
Widget elternWidget;
{
      Widget buttonW;

      /* XmPushButton zum Abspeichern einer Graphik erzeugen
       */
      buttonW = XtCreateManagedWidget ("speichern",
                                       xmPushButtonWidgetClass,
                                       elternWidget, (Arg*)NULL, 0);
      XtAddCallback (buttonW, XmNactivateCallback,
                     speichernCB, (XtPointer)NULL);
}


/*ARGSUSED*/
void unmanageCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Widget, das Callback aufrief, unmanagen
     */
    XtUnmanageChild (widget);
}


void setzeGraphikName (name)
String name;       /* neuer Name der Graphik */
{
    XmString label;        /* Compound-String fuer das Label */
    Arg      args[10];     /* Argumente-Liste */
    Cardinal n;            /* Lauf-Index */

    /* Alten Graphik-Namen freigeben und
     * neuen Namen setzen (falls nicht identisch)
     */
    if (graphikName != name) {
        XtFree (graphikName);
        graphikName = XtNewString (name);
    }

    /* Neues Label fuer Graphik-Name-Widget erzeugen, setzen
     * und freigeben
     */
    label = XmStringCreateLocalized (graphikName);
    n = 0;
    XtSetArg (args[n], XmNlabelString, label);  n++;
    XtSetValues (graphikNameWidget, args, n);
    XmStringFree (label);
}


Boolean ladeGraphik (dateiname)
String dateiname;    /* Dateiname fuer zu ladende Graphik */
{
    Pixmap       bitmap;          /* geladene Bitmap */
    int          status;          /* Laden geklappt ? */
    unsigned int breite, hoehe;   /* Groesse der geladenen Bitmap */
    int          dummyInt;        /* Dummy-Parameter */

    /* Bitmap-Datei aus 'dateiname' einlesen
     */
    status = XReadBitmapFile (display, graphikBitmap, dateiname,
                              &breite, &hoehe, &bitmap,
                              &dummyInt, &dummyInt);

    /* Hat das Laden geklappt ?
     */
    if (status != BitmapSuccess) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Datei nicht lesbar");
        return False;
    }

    /* Stimmt die Groesse ?
     */
    if (breite != GRAPHIK_BREITE || hoehe != GRAPHIK_HOEHE) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Datei hat falsche Groesse");
        return False;
    }

    /* Alte Bitmap freigeben und neue eintragen
     */
    XFreePixmap (display, graphikBitmap);
    graphikBitmap = bitmap;

    /* Inhalt ins Graphik-Fenster kopieren
     */
    XCopyPlane (display, graphikBitmap, XtWindow(graphikWidget), copyGC,
                0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE, 0, 0, 1);

    /* Dateinamen der geladenen Graphik setzen
     */
    setzeGraphikName (dateiname);

    return True;
}


/*ARGSUSED*/
void fsdFertigCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten mit Callback-Informationen */
{
    XmFileSelectionBoxCallbackStruct* FSBinfos;
    String                            dateiname;
    /* Aufruf-Daten der FileSelectionBox enthalten unter anderem
     * in der Komponente 'value' den eingegebenen Dateinamen
     */
    FSBinfos = (XmFileSelectionBoxCallbackStruct*)aufrufDaten;

    /* Dateinamen auslesen (Kopie mit eigenem Speicherplatz)
     */
    XmStringGetLtoR (FSBinfos->value, XmSTRING_DEFAULT_CHARSET,
                     &dateiname);

    /* Graphik dazu laden
     */
    ladeGraphik (dateiname);

    /* Speicherplatz fuer ausgelesenen Dateinamen wieder freigeben
     */
    XtFree (dateiname);
}


/*ARGSUSED*/
void ladenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Angezeigte Dateien auf aktuellen Stand bringen
     */
    XmFileSelectionDoSearch (fsdWidget, (XmString)NULL);

    /* FileSelectionDialog ausgeben
     */
    XtManageChild (fsdWidget);
}


void initFSD (elternWidget)
Widget elternWidget;
{
    Arg      args[10];
    Cardinal n;

    /* XmFileSelectionDialog erzeugen
     * (Rest des Programms wird bei Ausgabe gesperrt)
     */
    n = 0;
    XtSetArg (args[n], XmNdialogStyle,
                       XmDIALOG_FULL_APPLICATION_MODAL);  n++;
    fsdWidget = XmCreateFileSelectionDialog (elternWidget,
                                             "ladenDialog", args, n);

    /* Der Help-Button wird nicht verwendet
     */
    XtUnmanageChild (XmFileSelectionBoxGetChild (fsdWidget,
                                                 XmDIALOG_HELP_BUTTON));

    /* Callbacks fuer Eingabe-Ende und Eingabe-Abbruch installieren
     */
    XtAddCallback (fsdWidget, XmNokCallback,
                   fsdFertigCB, (XtPointer)NULL);
    XtAddCallback (fsdWidget, XmNokCallback,
                   unmanageCB, (XtPointer)NULL);
    XtAddCallback (fsdWidget, XmNcancelCallback,
                   unmanageCB, (XtPointer)NULL);
}


void initLadenButton (elternWidget)
Widget elternWidget;
{
      Widget buttonW;

      /* XmPushButton zum Laden einer Graphik erzeugen
       */
      buttonW = XtCreateManagedWidget ("laden", xmPushButtonWidgetClass,
                                      elternWidget, (Arg*)NULL, 0);
      XtAddCallback (buttonW, XmNactivateCallback,
                     ladenCB, (XtPointer)NULL);

      /* FileSelection-Dialog zum Laden erzeugen
       */
      initFSD (buttonW);
}


/*ARGSUSED*/
void hilfeCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    static Widget hilfeWidget = NULL;    /* Dialog-Widget */
    Arg           args[10];              /* Argumente-Liste */
    Cardinal      n;                     /* Lauf-Index */

    /* Falls noch nicht geschehen:
     *  XmMessageDialog-Widget erzeugen
     */
    if (hilfeWidget == NULL) {
        n = 0;
        XtSetArg (args[n], XmNautoUnmanage, True);              n++;
        XtSetArg (args[n], XmNdialogStyle, XmDIALOG_MODELESS);  n++;
        hilfeWidget = XmCreateMessageDialog (widget, "hilfeDialog",
                                             args, n);
 
        /* Cancel- und Help-Button werden nicht gebraucht
         */
        XtUnmanageChild (XmMessageBoxGetChild(hilfeWidget,
                                              XmDIALOG_HELP_BUTTON));
        XtUnmanageChild (XmMessageBoxGetChild(hilfeWidget,
                                              XmDIALOG_CANCEL_BUTTON));
    }
 
    /* XmMessageDialog-Widget ausgeben
     */
    XtManageChild (hilfeWidget);

    /* und (falls es verdeckt war) nach oben holen
     */
    {
      XWindowChanges werte;
      werte.stack_mode = Above;
      XReconfigureWMWindow (XtDisplay(XtParent(hilfeWidget)),
                            XtWindow(XtParent(hilfeWidget)),
                            XScreenNumberOfScreen(XtScreen(hilfeWidget)),
                            CWStackMode, &werte);
    }
}


Widget initMenueLeiste (elternWidget)
Widget elternWidget;
{
    Widget   menueLeisteW;
    Widget   dateiMenueW, buttonW;
    Arg      args[10];
    Cardinal n;

    /* XmMenuBar erzeugen
     */
    menueLeisteW = XmCreateMenuBar (elternWidget, "menueLeiste",
                                    (Arg*)NULL, 0);
    XtManageChild (menueLeisteW);

    /* Datei-Menue als XmPulldownMenu dazu erzeugen
     */
    dateiMenueW = XmCreatePulldownMenu (menueLeisteW, "dateiMenue",
                                        (Arg*)NULL, 0);
    
      /* Menuepunkt 'laden' erzeugen
       */
      initLadenButton (dateiMenueW);

      /* Menuepunkt 'speichern' erzeugen
       */
      initSpeichernButton (dateiMenueW);
    
      /* Menuepunkt 'speichernAls' erzeugen
       */
      initSpeichernAlsButton (dateiMenueW);
    
      /* Menuepunkt 'beenden' erzeugen
       */
      initBeendenButton (dateiMenueW);
    
    /* Menuepunkt 'datei' erzeugen, der das Datei-Menue aufruft
     */
    n = 0;
    XtSetArg (args[n], XmNsubMenuId, dateiMenueW);  n++;
    XtCreateManagedWidget ("datei", xmCascadeButtonWidgetClass,
                           menueLeisteW, args, n);

    /* Menuepunkt 'hilfe' erzeugen, der den Hilfetext ausgibt
     */
    buttonW = XtCreateManagedWidget ("hilfe", xmCascadeButtonWidgetClass,
                                     menueLeisteW, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNactivateCallback,
                   hilfeCB, (XtPointer)NULL);

    /* Menuepunkt 'hilfe' ist Help-Widget in der Menueleiste
     * (wird automatisch ganz rechts plaziert)
     */
    n = 0;
    XtSetArg (args[n], XmNmenuHelpWidget, buttonW);  n++;
    XtSetValues (menueLeisteW, args, n);

    /* Menueleiste zurueckliefern
     */
    return menueLeisteW;
}


void loescheGraphik()
{
   /* Graphik-Fenster und Graphik-Bitmap durch Bemalen mit
    * Hintergrundfarbe loeschen
    */
   XFillRectangle (display, XtWindow(graphikWidget), fensterGC[HG_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
   XFillRectangle (display, graphikBitmap, bitmapGC[HG_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
}


/*ARGSUSED*/
void loeschenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Graphik-Fenster und Graphik-Bitmap loeschen
     */
    loescheGraphik();
}


/*ARGSUSED*/
void invertierenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
   /* Graphik-Fenster und Graphik-Bitmap durch Bemalen mit
    * invertierendem GC invertieren
    */
   XFillRectangle (display, XtWindow(graphikWidget), fensterGC[INV_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
   XFillRectangle (display, graphikBitmap, bitmapGC[INV_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
}


/*ARGSUSED*/
void popupMenueAusgeben (widget, clientDaten, eventAdr,
                         weiterVerarbeitenAdr)
Widget    widget;               /* den Event-Handler aufrufendes Widget */
XtPointer clientDaten;          /* auszugebendes Popup-Menue */
XEvent*   eventAdr;             /* Zeiger auf ausloesenden X-Event */
Boolean*  weiterVerarbeitenAdr; /* weitere Event-Handler aufrufen ? */
{
    /* Nur Mausbutton 3 gibt Popup-Menue aus
     */
    if (eventAdr->xbutton.button != Button3) {
        return;
    }

    /* Popup-Menue anhand Mausposition positionieren
     */
    XmMenuPosition ((Widget)clientDaten, &eventAdr->xbutton);

    /* Popup-Menue ausgeben
     */
    XtManageChild ((Widget)clientDaten);
}


void initGraphikPopup (elternWidget)
Widget elternWidget;
{
    Widget popupMenueW, buttonW;

    /* XmPopupMenu-Widget erzeugen
     */
    popupMenueW = XmCreatePopupMenu (elternWidget, "graphikPopup",
                                     (Arg*)NULL, 0);

    /* Event-Handler zum Ausgeben des Popup-Menues installieren.
     * Das auszugebende Popup-Menue wird als Client-Datum uebergeben.
     */
    XtAddEventHandler (elternWidget, ButtonPressMask, False,
                       popupMenueAusgeben, (XtPointer)popupMenueW);

    /* XmPushButton zum Invertieren der Graphik erzeugen
     */
    buttonW = XtCreateManagedWidget ("graphikInvers",
                                     xmPushButtonWidgetClass,
                                     popupMenueW, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNactivateCallback,
                   invertierenCB, (XtPointer)NULL);

    /* XmPushButton zum Loeschen der Graphik erzeugen
     */
    buttonW = XtCreateManagedWidget ("graphikLoeschen",
                                     xmPushButtonWidgetClass,
                                     popupMenueW, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNactivateCallback,
                   loeschenCB, (XtPointer)NULL);
}


/*ARGSUSED*/
void exposeCB (widget, clientDaten, aufrufDaten)
Widget    widget;        /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten mit Callback-Informationen */
{
    XmDrawingAreaCallbackStruct* DAinfos;

    /* Callback-Informationen mit Event-Struktur
     */
    DAinfos = (XmDrawingAreaCallbackStruct*)aufrufDaten;

    /* Im Expose-Event angegebenen Fenster-Ausschnitt 
     * von Graphik-Bitmap ins Graphik-Fenster kopieren
     */
    XCopyPlane (display, graphikBitmap, XtWindow(widget), copyGC,
                DAinfos->event->xexpose.x, DAinfos->event->xexpose.y,
                DAinfos->event->xexpose.width,
                DAinfos->event->xexpose.height,
                DAinfos->event->xexpose.x, DAinfos->event->xexpose.y,
                1);
}


/*ARGSUSED*/
void motionEventHandler (widget, clientDaten, eventAdr,
                         weiterVerarbeitenAdr)
Widget    widget;               /* den Event-Handler aufrufendes Widget */
XtPointer clientDaten;          /* Zeiger auf vorherige Koordinaten */
XEvent*   eventAdr;             /* Zeiger auf ausloesenden X-Event */
Boolean*  weiterVerarbeitenAdr; /* weitere Event-Handler aufrufen ? */
{
    int x_von, y_von, x_nach, y_nach;

    /* Maus-Koordinaten auslesen
     */
    x_nach = eventAdr->xmotion.x;
    y_nach = eventAdr->xmotion.y;

    if (!kontinuierlich) {
        /* Nur den Punkt aus dem Motion-Event malen
         */
        x_von = x_nach;
        y_von = y_nach;
    }
    else {
        /* Vorherige Koordinaten auslesen
         */
        x_von = ((XPoint*)clientDaten)->x;
        y_von = ((XPoint*)clientDaten)->y;

        if (malmodus == INV_MODUS) {
            /* Da erster Punkt bereits invertiert ist und
             * nochmal invertiert wird, hebt sich dies auf.
             * Deswegen wird er hier nochmal invertiert.
             */
            XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
                       x_von, y_von, x_von, y_von);
            XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
                       x_von, y_von, x_von, y_von);
        }
    }

    /* Neue Linie bzw. neuen Punkt malen
     */
    XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
               x_von, y_von, x_nach, y_nach);
    XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
               x_von, y_von, x_nach, y_nach);

    /* Neue Koordinaten als 'vorherige Koordinaten' merken
     */
    ((XPoint*)clientDaten)->x = x_nach;
    ((XPoint*)clientDaten)->y = y_nach;
}


/*ARGSUSED*/
void inputCB (widget, clientDaten, aufrufDaten)
Widget    widget;           /* den Callback aufrufendes Widget */
XtPointer clientDaten;      /* Zeiger auf alte Koordinaten */
XtPointer aufrufDaten;      /* Callback-Informationen */
{
    XmDrawingAreaCallbackStruct* DAinfos;
    int                          x, y;

    /* Callback-Informationen mit Event-Struktur
     */
    DAinfos = (XmDrawingAreaCallbackStruct*)aufrufDaten;

    /* Reaktion nur auf Druecken von Mausbutton 1
     */
    if (DAinfos->event->type == ButtonPress &&
        DAinfos->event->xbutton.button == Button1) {
          /* Koordinaten auslesen
           */
          x = DAinfos->event->xbutton.x;
          y = DAinfos->event->xbutton.y;

          /* Ersten Punkt malen
           * (muss als Linie gemalt werden, da sonst Breite 1)
           */
          XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
                     x, y, x, y);
          XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
                     x, y, x, y);
          /* Start-Koordinaten als neue "vorherige Koordinaten"
           * merken
           */
          ((XPoint*)clientDaten)->x = x;
          ((XPoint*)clientDaten)->y = y;
    }
}


void initGCs (widget)
Widget widget;        /* Widget, fuer das die GCs gebraucht werden */
{
    Pixel         fg, bg;     /* Farbwerte fuer Vorder- und Hintergrund */
    XGCValues     werte;      /* Struktur zum Setzen von GC-Attributen */
    unsigned long maske;      /* Maske fuer GC-Attribute */
    Window        root;       /* Root-Window */
    Arg           args[10];   /* Argumente-Liste */
    Cardinal      n;          /* Lauf-Index */

    /* Farbwerte des Graphik-Widgets auslesen
     * (es geht auch rot auf gruen)
     */
    n = 0;
    XtSetArg (args[n], XmNforeground, &fg);  n++;
    XtSetArg (args[n], XmNbackground, &bg);  n++;
    XtGetValues (widget, args, n);

    /* GCs fuer das Graphik-Fenster und die Graphik-Bitmap erzeugen.
     * - Fuer jeden Mal-Modus einen GC.
     * - Da das Fenster zum Widget auch mehrere Farbebenen haben kann,
     *   muessen die GCs von Fenster und Bitmap verschieden sein.
     * - Da in den GCs jeweils die Linienbreite geaendert wird,
     *   muessen es aenderbare sein (aber der Font ist unerheblich).
     * - Zum Kopieren der Bitmap-Daten in das Fenster bei Expose-Events
     *   kann ein nicht veraenderbarer GC verwendet werden.
     */
    
    /* Da das Graphik-Widget noch nicht realisiert ist, wird das
     * Root-Window beim Erzeugen der GCs als Drawable, das
     * Bildschirm und Anzahl der Farbebenen des Graphik-Fensters
     * identifiziert, verwendet.
     */
    root = RootWindowOfScreen(XtScreen(widget));
    /* Runde Linienenden
     */
    werte.cap_style = CapRound;

    /* GCs fuer das Malen mit Vordergrund-Farbwert erzeugen
     */
    werte.foreground = fg;
    maske = GCForeground|GCCapStyle;
    fensterGC[VG_MODUS] = XtAllocateGC (widget, 0, maske, &werte,
                                        GCLineWidth, GCFont);
    werte.foreground = 1;
    bitmapGC[VG_MODUS]  = XCreateGC (display, graphikBitmap,
                                     maske, &werte);

    /* GCs fuer invertierendes Malen erzeugen
     */
    werte.foreground = fg^bg;
    werte.function = GXxor;
    maske = GCForeground|GCFunction|GCCapStyle;
    fensterGC[INV_MODUS] = XtAllocateGC (widget, 0, maske, &werte,
                                         GCLineWidth, GCFont);
    werte.foreground = 1^0;
    bitmapGC[INV_MODUS]  = XCreateGC (display, graphikBitmap,
                                      maske, &werte);

    /* GCs fuer das Malen mit Hintergrund-Farbwert erzeugen
     */
    werte.foreground = bg;
    maske = GCForeground|GCCapStyle;
    fensterGC[HG_MODUS] = XtAllocateGC (widget, 0, maske, &werte,
                                        GCLineWidth, GCFont);
    werte.foreground = 0;
    bitmapGC[HG_MODUS] =  XCreateGC (display, graphikBitmap,
                                     maske, &werte);

    /* GC zum Kopieren der Bitmap-Daten ins Graphik-Fenster erzeugen
     * (read-only)
     */
    werte.foreground = fg;
    werte.background = bg;
    maske = GCForeground|GCBackground;
    copyGC = XtGetGC (widget, maske, &werte);
}


void initGraphikWidget (elternWidget)
Widget elternWidget;
{
    static XPoint alteKoords;     /* jeweils vorherige Koordinaten *
                                   * (dient als Client-Datum)      */
    Arg           args[10];       /* Argumente-Liste */
    Cardinal      n;              /* Lauf-Index */

    /* XmDrawingArea als Graphik-Widget erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNwidth,  GRAPHIK_BREITE);  n++;
    XtSetArg (args[n], XmNheight, GRAPHIK_HOEHE);   n++;
    graphikWidget = XtCreateManagedWidget ("graphik",
                                           xmDrawingAreaWidgetClass,
                                           elternWidget, args, n);

    /* Input-Callback installieren
     */
    XtAddCallback (graphikWidget, XmNinputCallback,
                   inputCB, (XtPointer)&alteKoords);

    /* Event-Handler fuer Motion-Events installieren
     */
    XtAddEventHandler (graphikWidget, Button1MotionMask, False,
                       motionEventHandler, (XtPointer)&alteKoords);

    /* Expose-Callback installieren
     */
    XtAddCallback (graphikWidget, XmNexposeCallback,
                   exposeCB, (XtPointer)NULL);

    /* Graphik-Bitmap fuer Kopie des Fensterinhaltes erzeugen
     */
    graphikBitmap = XCreatePixmap (display,
                            RootWindowOfScreen(XtScreen(graphikWidget)),
                            GRAPHIK_BREITE, GRAPHIK_HOEHE, 1);

    /* GCs zum Malen in Graphik-Fenster und Graphik-Bitmap erzeugen
     */
    initGCs (graphikWidget);
    
    /* Popup-Menue fuer das Graphik-Widget erzeugen
     */
    initGraphikPopup (graphikWidget);
}


/*ARGSUSED*/
void kontinuierlichCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten mit Callback-Informationen */
{
    /* Davon betroffene Variable 'kontinuierlich' entsprechend
     * neuem Zustand des Buttons (Komponente 'set' in den
     * Callback-Informationen) setzen.
     */
    kontinuierlich = ((XmToggleButtonCallbackStruct*)aufrufDaten)->set;
}


void initKontinuierlichButton (elternWidget)
Widget elternWidget;
{
    Widget   buttonW;
    Arg      args[10];
    Cardinal n;

    /* XmToggleButton fuer kontinuierliches Malen erzeugen
     */
    buttonW = XtCreateManagedWidget ("kontinuierlich",
                                     xmToggleButtonWidgetClass,
                                     elternWidget, (Arg*)NULL, 0);
    XtAddCallback (buttonW, XmNvalueChangedCallback,
                   kontinuierlichCB, (XtPointer)NULL);

    /* Default-Zustand (an oder aus) kann von aussen als Ressource
     * gesetzt werden.
     * Dieser muss ausgelesen werden, damit die davon betroffene
     * Variable "kontinuierlich" entsprechend gesetzt wird.
     */
    n = 0;
    XtSetArg (args[n], XmNset, &kontinuierlich);  n++;
    XtGetValues (buttonW, args, n);    
}


/*ARGSUSED*/
void linienbreiteCB (widget, clientDaten, aufrufDaten)
Widget   widget;            /* den Calllback aufrufendes Widget */
XtPointer clientDaten;      /* Client-Daten (hier Linienbreite) */
XtPointer aufrufDaten;      /* Aufruf-Daten (hier nicht verwendet) */
{
    XGCValues werte;      /* Struktur zum Setzen von GC-Attributen */
    int       i;          /* Laufvariable */

    /* Client-Datum enthaelt Linienbreite
     */
    werte.line_width = (int)clientDaten;
    /* Fuer jeden Mal-Modus die GCs anpassen
     */
    for (i=0; i<MODUS_ANZ; i++) {
        XChangeGC (display, fensterGC[i], GCLineWidth, &werte);
        XChangeGC (display, bitmapGC[i],  GCLineWidth, &werte);
    }
}


void initOptionenMenue (elternWidget)
Widget elternWidget;
{
    Widget   optionenW, buttonW, optionsMenueW;
    Arg      args[10];
    Cardinal n;

    /* XmPulldownMenu mit Optionen erzeugen
     */
    optionenW = XmCreatePulldownMenu (elternWidget, "optionen",
                                     (Arg*)NULL, 0);

      /* Option "schmal" (Linienbreite 0)
       */
      buttonW = XtCreateManagedWidget ("schmal", xmPushButtonWidgetClass,
                                       optionenW, (Arg*)NULL, 0);
      XtAddCallback (buttonW, XmNactivateCallback,
                     linienbreiteCB, (XtPointer)0);

      /* Option "mittel" (Linienbreite 5)
       */
      buttonW = XtCreateManagedWidget ("mittel", xmPushButtonWidgetClass,
                                       optionenW, (Arg*)NULL, 0);
      XtAddCallback (buttonW, XmNactivateCallback,
                     linienbreiteCB, (XtPointer)5);

      /* Option "breit" (Linienbreite 9)
       */
      buttonW = XtCreateManagedWidget ("breit", xmPushButtonWidgetClass,
                                       optionenW, (Arg*)NULL, 0);
      XtAddCallback (buttonW, XmNactivateCallback,
                     linienbreiteCB, (XtPointer)9);    

    /* XmOptionMenu mit Optionen erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNsubMenuId, optionenW);  n++;
    optionsMenueW = XmCreateOptionMenu (elternWidget, "breitenMenue",
                                        args, n);
    XtManageChild (optionsMenueW);
}


/*ARGSUSED*/
void malmodusCB (widget, clientDaten, aufrufDaten)
Widget    widget;         /* den Callback aufrufendes Widget */
XtPointer clientDaten;    /* Client-Daten (hier Mal-Modus) */
XtPointer aufrufDaten;    /* Aufruf-Daten mit Callback-Informationen */
{
    /* Falls XmToggleButton gesetzt wurde, dessen Mal-Modus
     * eintragen (wird als Client-Datum uebergeben)
     */
    if (((XmToggleButtonCallbackStruct*)aufrufDaten)->set) {
        malmodus = (int)clientDaten;
    }
}


void initRadioBox (elternWidget)
Widget elternWidget;
{
    Widget   radioBoxW, buttonW;
    Arg      args[10];
    Cardinal n;

    /* XmRadioBox fuer Mal-Modus erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNentryClass, xmToggleButtonWidgetClass);  n++;
    radioBoxW = XmCreateRadioBox (elternWidget, "malmodusBox", args, n);
    XtManageChild (radioBoxW);

    /* XmToggleButton fuer Malen mit Vordergrund-Farbwert (Default)
     */
    n = 0;
    XtSetArg (args[n], XmNset, True);  n++;
    buttonW = XtCreateManagedWidget ("vordergrund",
                                     xmToggleButtonWidgetClass,
                                     radioBoxW, args, n);
    XtAddCallback (buttonW, XmNvalueChangedCallback,
                   malmodusCB, (XtPointer)VG_MODUS);
    malmodus = VG_MODUS;

    /* XmToggleButton fuer invertierendes Malen
     * (Vordergrund-Farbwert ^ Hintergrund-Farbwert)
     */
    n = 0;
    XtSetArg (args[n], XmNset, False);  n++;
    buttonW = XtCreateManagedWidget ("invertieren",
                                     xmToggleButtonWidgetClass,
                                     radioBoxW, args, n);
    XtAddCallback (buttonW, XmNvalueChangedCallback,
                   malmodusCB, (XtPointer)INV_MODUS);

    /* XmToggleButton fuer Malen mit Hintergrund-Farbwert
     */
    n = 0;
    XtSetArg (args[n], XmNset, False);  n++;
    buttonW = XtCreateManagedWidget ("hintergrund",
                                     xmToggleButtonWidgetClass,
                                     radioBoxW, args, n);
    XtAddCallback (buttonW, XmNvalueChangedCallback,
                   malmodusCB, (XtPointer)HG_MODUS);
}


void initAttributeBoxWidgets (elternWidget)
Widget elternWidget;
{
    /* Radio-Box fuer Mal-Modus erzeugen
     */
    initRadioBox (elternWidget);

    /* XmSeparator-Widget erzeugen
     */
    XtCreateManagedWidget ("separator", xmSeparatorWidgetClass,
                           elternWidget, (Arg*)NULL, 0);

    /* Optionen-Menue fuer Linienbreite erzeugen
     */
    initOptionenMenue (elternWidget);

    /* XmSeparator-Widget erzeugen
     */

    XtCreateManagedWidget ("separator", xmSeparatorWidgetClass,
                           elternWidget, (Arg*)NULL, 0);

    /* Toggle-Button fuer 'kontinuierlich' erzeugen
     */
    initKontinuierlichButton (elternWidget);
}


Widget initForm (elternWidget)
Widget elternWidget;
{
    Widget   formW, dateinameW, attributeBoxW, scrollW;
    Arg      args[10];
    Cardinal n;

    /* XmForm-Widget als Container erzeugen
     */
    formW = XtCreateManagedWidget ("form", xmFormWidgetClass,
                                   elternWidget, (Arg*)NULL, 0);

    /* Label fuer 'Dateiname:' erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNtopAttachment,  XmATTACH_FORM);  n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM);  n++;
    dateinameW = XtCreateManagedWidget ("dateiname",
                                        xmLabelWidgetClass,
                                        formW, args, n);

    /* Label fuer aktuellen Graphik-Namen erzeugen.
     * Wird mit 'setzeGraphikName()' jeweils gesetzt.
     */
    n = 0;
    XtSetArg (args[n], XmNtopAttachment,   XmATTACH_FORM);    n++;
    XtSetArg (args[n], XmNleftAttachment,  XmATTACH_WIDGET);  n++;
    XtSetArg (args[n], XmNleftWidget,      dateinameW);       n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM);    n++;
    graphikNameWidget = XtCreateManagedWidget ("graphikName",
                                               xmLabelWidgetClass,
                                               formW, args, n);

    /* Attribute-Box erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNtopAttachment,  XmATTACH_WIDGET);  n++;
    XtSetArg (args[n], XmNtopWidget,      dateinameW);       n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM);    n++;
    attributeBoxW = XtCreateManagedWidget ("attributeBox",
                                          xmRowColumnWidgetClass,
                                          formW, args, n);

    /* Unterwidgets der Kommando-Box erzeugen
     */
    initAttributeBoxWidgets (attributeBoxW);

    /* XmScrolledWindow fuer Graphik-Verwaltung erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNtopAttachment,    XmATTACH_WIDGET);   n++;
    XtSetArg (args[n], XmNtopWidget,        dateinameW);        n++;
    XtSetArg (args[n], XmNleftAttachment,   XmATTACH_WIDGET);   n++;
    XtSetArg (args[n], XmNleftWidget,       attributeBoxW);     n++;
    XtSetArg (args[n], XmNrightAttachment,  XmATTACH_FORM);     n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM);     n++;
    scrollW = XtCreateManagedWidget ("scrolledGraphik",
                                     xmScrolledWindowWidgetClass,
                                     formW, args, n);

    /* Graphik-Widget erzeugen
     */
    initGraphikWidget (scrollW);

    /* Form-Widget zurueckliefern
     */
    return formW;
}


void initWidgetBaum (applShell)
Widget applShell;              /* Applikations-Shell */
{
    Widget mainW, formW, menueW;   /* temporaere Widgets */

    /* XmMainWindow-Widget erzeugen
     */
    mainW = XtCreateManagedWidget ("main", xmMainWindowWidgetClass,
                                   applShell, (Arg*)NULL, 0);

    /* Menueleiste dafuer erzeugen
     */
    menueW = initMenueLeiste (mainW);

    /* Form-Widget als Arbeitsbereich dafuer erzeugen
     */
    formW = initForm (mainW);

    /* Komponenten des XmMainWindow-Widgets entsprechend setzen
     */
    XmMainWindowSetAreas (mainW, menueW, (Widget)NULL,
                          (Widget)NULL, (Widget)NULL, formW);
}


void falscheParameter (argcFalsch, argvFalsch)
int   argcFalsch;       /* Anzahl falscher Parameter */
char* argvFalsch[];     /* Array  falscher Parameter */
{
    int    zeichenAnzahl;
    String warnung;
    int    i;

    /* Anzahl der Zeichen fuer den Warnungs-Text ermitteln und
     * entsprechenden Speicherplatz anfordern
     */
    zeichenAnzahl = 0;
    for (i=0; i<argcFalsch; i++) {
        zeichenAnzahl += strlen(argvFalsch[i]) + 1;
    }
    warnung = XtMalloc ((zeichenAnzahl+30) * sizeof(char));

    /* Warnungs-Text "Falsche Parameter: " eintragen
     */
    strcpy (warnung, "Falsche Parameter:");
    for (i=0; i<argcFalsch; i++) {
        strcat (warnung, " ");
        strcat (warnung, argvFalsch[i]);
    }

    /* Warnung ausgeben
     */
    XtAppWarning (kontext, warnung);
    
    /* Angeforderten Speicherplatz wieder freigeben
     */
    XtFree (warnung);
}


void liesApplikationsRessourcen (applShell)
Widget applShell;
{
   /* Variable fuer Applikations-Ressourcen
    */
   ApplResTyp applRessourcen;

   /* Applikations-Ressourcen auslesen
    */
   XtGetApplicationResources (applShell, &applRessourcen,
                              ressourcen, XtNumber(ressourcen),
                              (Arg*)NULL, 0);

   /* Applikations-Ressource 'datei' verarbeiten
    * (Name der ersten Bitmap-Datei)
    */
   setzeGraphikName (applRessourcen.datei);

   /* Applikations-Ressource 'appDefaultsVersion' verarbeiten
    * (Version der Application-Defaults)
    */
   if (applRessourcen.version == 0) {
       /* Default-Wert, also keine Appl.-Defaults installiert
        */
       XtAppWarning (kontext, "Keine Application-Defaults installiert");
   }
   else if (applRessourcen.version != APP_DEFAULTS_VERSION) {
       /* Falscher Wert, also falsche Appl.-Defaults installiert
        */
       XtAppWarning (kontext, "Falsche Application-Defaults installiert");
   }
}


main (argc, argv)
int   argc;
char* argv[];
{
    Widget   applShell;      /* Applikations-Shell */

    /* Initialisierungen und Applikations-Shell erzeugen
     */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 optionen, XtNumber(optionen),
                                 &argc, argv,
                                 fallbackRessourcen,
                                 (Arg*)NULL, 0);

    /* Warnung fuer nicht verarbeitete Parameter ausgeben
     */
    if (argc != 1) {
        falscheParameter (argc-1, argv+1);
    }
    /* Display-Variable initialisieren
     */
    display = XtDisplay (applShell);

#if XmVersion <= 1002
    /* damit auch in Motif 1.2 Tear-Off-Menues moeglich sind:
     */
    XmRepTypeInstallTearOffModelConverter();
#endif

    /* Widgetbaum unter der Applikations-Shell erzeugen
     */
    initWidgetBaum (applShell);

    /* Applikations-Ressourcen auslesen und verarbeiten
     */
    liesApplikationsRessourcen (applShell);

    /* Widgetbaum realisieren
     */
    XtRealizeWidget (applShell);

    /* Graphik leeren
     */
    loescheGraphik();

    /* Falls vorhanden und lesbar, erste Graphik laden
     */
    if (access(graphikName,R_OK) == 0) {
        ladeGraphik (graphikName);
    }

    /* Hauptschleife
     */
    XtAppMainLoop (kontext);
}

